<?php

class Maarch_Sniffs_Functions_FunctionDeclarationArgumentSpacingSniff 
    implements PHP_CodeSniffer_Sniff
{


    /**
     * Returns an array of tokens this test wants to listen for.
     *
     * @return array
     */
    public function register()
    {
        return array(T_FUNCTION);

    }//end register()


    /**
     * Processes this test, when one of its tokens is encountered.
     *
     * @param PHP_CodeSniffer_File $phpcsFile The file being scanned.
     * @param int                  $stackPtr  The position of the current token 
     * 										  in the stack passed in $tokens.
     *
     * @return void
     */
    public function process(PHP_CodeSniffer_File $phpcsFile, $stackPtr)
    {
        $tokens = $phpcsFile->getTokens();

        $functionName = $phpcsFile->findNext(array(T_STRING), $stackPtr);
        $openBracket  = $tokens[$stackPtr]['parenthesis_opener'];
        $closeBracket = $tokens[$stackPtr]['parenthesis_closer'];
        $multiLine = ($tokens[$openBracket]['line'] !== $tokens[$closeBracket]['line']);
        
        $firstArg = $phpcsFile->findNext(array(T_WHITESPACE, T_COMMENT), 
                                         $openBracket + 1, 
                                         $closeBracket + 1, 
                                         true);
        
        // there must be no space between empty parentheseses 
        if (($firstArg === $closeBracket) 
            && (($closeBracket - $openBracket) !== 1)
        ) {
            $error = 'Expected no spaces between brackets of function '
                   . 'declaration.';
            $phpcsFile->addError($error, $openBracket + 1, 
            					 "SpaceInEmptyFunctionDeclaration");
            return;
        }
        
        // there must be no space after opening parenthesis
        if ($tokens[$openBracket + 1]['code'] === T_WHITESPACE) {
            $nextArg = $phpcsFile->findNext(
                array(T_VARIABLE), 
                $openBracket + 2,
                $closeBracket
            );
            if ($tokens[$nextArg]['line'] === $tokens[$openBracket + 1]['line']) {
                $error = "Opening parenthesis must not be followed by space";
                $phpcsFile->addError($error, $openBracket + 1, 
                					 "NoSpaceAfterOpenBracket");
            }
        }
        
        // there must be no space before closing parenthesis
        if (!$multiLine
            && ($tokens[$closeBracket - 1]['code'] === T_WHITESPACE)
         ) {
            $error = "Closing parenthesis must not be preceded by space";
            $phpcsFile->addError($error, $closeBracket - 1, 
            					 "NoSpaceBeforeClosingBracket");
        } 

        $currentToken = $openBracket;
        $params    = array();
        while (
            ($currentToken = $phpcsFile->findNext(
                array(T_EQUAL, T_COMMA), 
                ($currentToken + 1), $closeBracket
            )) !== false
        ) {
            
            $currentTokenCode = $tokens[$currentToken]['code'];
            $nextToken = $currentToken + 1;
            $prevToken = $currentToken - 1;
            $nextArg = $phpcsFile->findNext(
                array(T_VARIABLE), 
                $currentToken + 1,
                $closeBracket
            );
            if ($tokens[$nextArg]['line'] !== $tokens[$currentToken]['line']) {
                $isEndOfLine = true;
            } else {
                $isEndOfLine = false;
            }
            
            
            if ($currentTokenCode === T_COMMA) {
                
                //Check for space after comma
                if (!$isEndOfLine) {
                    if ($tokens[$nextToken]['code'] !== T_WHITESPACE) {
                        $error = "Comma must be followed by a single space";
                        $phpcsFile->addError($error, $nextToken, 
                        					 "NoSpaceAfterComma");
                    } else {
                        $spaceLen = strlen($tokens[$nextToken]['content']);
                        if ($spaceLen !== 1) {
                            $error = "Comma must be followed by a single space; "
                                   . "expected 1, found %s";
                            $data = array($spaceLen);
                            $phpcsFile->addError($error, $nextToken, 
                            					 "SpaceAfterComma", $data);
                        }
                    }
                }
            } elseif ($currentTokenCode === T_EQUAL) {
                //check for spaces around comma
                if ($tokens[$nextToken]['code'] === T_WHITESPACE) {
                    $error = "Equal sign must not be followed by space";
                    $phpcsFile->addError($error, $nextToken, 
                    					 "NoSpaceAfterEqual");
                }
                if ($tokens[$prevToken]['code'] === T_WHITESPACE) {
                    $error = "Equal sign must not be preceded by space";
                    $phpcsFile->addError($error, $prevToken, 
                    					 "NoSpaceBeforeEqual");
                }
            }

        }//end while

    }//end process()


}